from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import (
    AccountDepositBody,
    AccountLimitsBody,
    AccountTransferBody,
    AccountWithdrawBody,
    BalanceHoldBody,
    Contact,
    EwalletIdContactsBody,
    EwalletsEwalletTokenBody,
    InlineResponse200_100,
    InlineResponse200_101,
    InlineResponse200_102,
    InlineResponse200_103,
    InlineResponse200_66,
    InlineResponse200_67,
    InlineResponse200_68,
    InlineResponse200_69,
    InlineResponse200_70,
    InlineResponse200_71,
    InlineResponse200_72,
    InlineResponse200_73,
    InlineResponse200_97,
    InlineResponse200_98,
    InlineResponse200_99,
    TransferResponseBody,
    UpdateEwalletStatusStatus,
    V1EwalletsBody,
)


class EWalletsService(BaseService):

    @cast_models
    def funds_transfer(
        self,
        request_body: AccountTransferBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_67:
        """Transfer funds between Rapyd Wallets.

        :param request_body: The request body.
        :type request_body: AccountTransferBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Transacton properties
        :rtype: InlineResponse200_67
        """

        Validator(AccountTransferBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/account/transfer", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_67._unmap(response)

    @cast_models
    def set_funds_transfer_response(
        self,
        request_body: TransferResponseBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_67:
        """Respond to a funds transfer between wallets. The transferee uses this method to accept or decline the transfer.

        :param request_body: The request body.
        :type request_body: TransferResponseBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Transaction properties
        :rtype: InlineResponse200_67
        """

        Validator(TransferResponseBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/account/transfer/response",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_67._unmap(response)

    @cast_models
    def add_funds_to_wallet_account(
        self,
        request_body: AccountDepositBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_68:
        """Transfer virtual currency to a Rapyd Wallet account.<BR>If the account does not already exist for the indicated currency, it is created.<BR>Use this method in the sandbox for testing purposes.<BR>This method triggers the 'Funds Added Webhook'.

        :param request_body: The request body.
        :type request_body: AccountDepositBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Returns details on the funds that were added to the account.
        :rtype: InlineResponse200_68
        """

        Validator(AccountDepositBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/account/deposit", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_68._unmap(response)

    @cast_models
    def details_of_add_fundsto_wallet_account(
        self,
        id_: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_68:
        """Retrieve details of a specific 'Add Funds to Wallet Account' request.<BR> Use this method in the sandbox for testing purposes.

        :param id_: ID of the 'Add Funds to Wallet Account' request, from the `id` field in the data object of the response. UUID.
        :type id_: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Returns details of an 'Add Funds' Request.
        :rtype: InlineResponse200_68
        """

        Validator(str).validate(id_)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/account/deposit/{{id}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_68._unmap(response)

    @cast_models
    def withdraw_funds_from_wallet_account(
        self,
        request_body: AccountWithdrawBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_68:
        """Remove virtual currency from a Rapyd Wallet account.<BR>If the account does not have sufficient funds in the indicated currency, the funds transfer fails.<BR>Use this method in the sandbox for testing purposes.<BR>This method triggers the 'Funds Removed' Webhook

        :param request_body: The request body.
        :type request_body: AccountWithdrawBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Returns details on the funds that were removed from the account.
        :rtype: InlineResponse200_68
        """

        Validator(AccountWithdrawBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/account/withdraw", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_68._unmap(response)

    @cast_models
    def get_details_of_remove_funds_request(
        self,
        id_: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_68:
        """Retrieve details of a specific 'Remove Funds From Wallet Account' request.<BR>Use this method in the sandbox for testing purposes.

        :param id_: ID of the 'Remove Funds From Wallet Account' request, from the `id` field in the `data` object of the response. UUID.
        :type id_: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Returns details on the funds that were removed from the account via a specific request.
        :rtype: InlineResponse200_68
        """

        Validator(str).validate(id_)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/account/withdraw{{id}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_68._unmap(response)

    @cast_models
    def put_funds_on_hold(
        self,
        request_body: BalanceHoldBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_69:
        """Put a hold on funds in the wallet. Sandbox only.<BR> This action transfers funds from the available balance to the on-hold balance. If the wallet does not have enough funds in the available balance in the specified currency, the transfer fails.<BR>This method triggers the **Transfer Funds Between Balances** webhook. This webhook contains the same information as the response.<BR>The customer cannot move funds that are on hold until the client releases the hold.<BR>This method is relevant for **person** and **company** wallets.

        :param request_body: The request body.
        :type request_body: BalanceHoldBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Funds put on hold.
        :rtype: InlineResponse200_69
        """

        Validator(BalanceHoldBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/ewallets/accounts/balance/hold",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_69._unmap(response)

    @cast_models
    def get_ewallet_contacts(
        self,
        ewallet_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_70:
        """Retrieve all contacts for a wallet.

        :param ewallet_id: ID of the Rapyd Wallet that this contact is associated with. String starting with **ewallet_**.
        :type ewallet_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List Contacts for a Rapyd Wallet,
        :rtype: InlineResponse200_70
        """

        Validator(str).validate(ewallet_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/ewallets/{{ewalletId}}/contacts",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("ewalletId", ewallet_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_70._unmap(response)

    @cast_models
    def create_ewallet_contact(
        self,
        request_body: EwalletIdContactsBody,
        ewallet_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_71:
        """Add a personal contact to a company wallet or client wallet.

        :param request_body: The request body.
        :type request_body: EwalletIdContactsBody
        :param ewallet_id: ID of the Rapyd Wallet that this contact is associated with. String starting with **ewallet_**.
        :type ewallet_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List Contacts for a Rapyd Wallet,
        :rtype: InlineResponse200_71
        """

        Validator(EwalletIdContactsBody).validate(request_body)
        Validator(str).validate(ewallet_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/ewallets/{{ewalletId}}/contacts",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("ewalletId", ewallet_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_71._unmap(response)

    @cast_models
    def get_ewallet_contact(
        self,
        ewallet_id: str,
        contact_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_71:
        """Retrieve a contact for an existing Rapyd Wallet.

        :param ewallet_id: ID of the Rapyd Wallet that this contact is associated with. String starting with **ewallet_**.
        :type ewallet_id: str
        :param contact_id: One of two values. either ID of the contact - String starting with the prefix **cont_** or Contact reference ID.
        :type contact_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Retrieve a contact for an existing Rapyd Wallet.
        :rtype: InlineResponse200_71
        """

        Validator(str).validate(ewallet_id)
        Validator(str).validate(contact_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/ewallets/{{ewalletId}}/contacts/{{contactId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("ewalletId", ewallet_id)
            .add_path("contactId", contact_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_71._unmap(response)

    @cast_models
    def update_ewallet_contact(
        self,
        request_body: Contact,
        ewallet_id: str,
        contact_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_71:
        """Update a contact for a Rapyd Wallet.

        :param request_body: The request body.
        :type request_body: Contact
        :param ewallet_id: ID of the Rapyd Wallet that this contact is associated with. String starting with **ewallet_**.
        :type ewallet_id: str
        :param contact_id: ID of the contact. String starting with the prefix **cont_**.
        :type contact_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Retrieve a contact for an existing Rapyd Wallet.
        :rtype: InlineResponse200_71
        """

        Validator(Contact).validate(request_body)
        Validator(str).validate(ewallet_id)
        Validator(str).validate(contact_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/ewallets/{{ewalletId}}/contacts/{{contactId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("ewalletId", ewallet_id)
            .add_path("contactId", contact_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_71._unmap(response)

    @cast_models
    def delete_ewallet_contact(
        self,
        ewallet_id: str,
        contact_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_72:
        """Delete a personal contact from a company wallet or client wallet.

        :param ewallet_id: ID of the Rapyd Wallet that this contact is associated with. String starting with **ewallet_**.
        :type ewallet_id: str
        :param contact_id: ID of the contact. String starting with the prefix **cont_**.
        :type contact_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Retrieve a contact for an existing Rapyd Wallet.
        :rtype: InlineResponse200_72
        """

        Validator(str).validate(ewallet_id)
        Validator(str).validate(contact_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/ewallets/{{ewalletId}}/contacts/{{contactId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("ewalletId", ewallet_id)
            .add_path("contactId", contact_id)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_72._unmap(response)

    @cast_models
    def get_ewallet_contact_compliance_levels(
        self,
        ewallet_id: str,
        contact_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_73:
        """Verify the compliance status of a personal contact.

        :param ewallet_id: ID of the Rapyd Wallet that this contact is associated with. String starting with **ewallet_**.
        :type ewallet_id: str
        :param contact_id: ID of the contact. String starting with **cont_**.
        :type contact_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Verify the compliance status of a personal contact.
        :rtype: InlineResponse200_73
        """

        Validator(str).validate(ewallet_id)
        Validator(str).validate(contact_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/ewallets/{{ewalletId}}/contacts/{{contactId}}/compliance_levels",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("ewalletId", ewallet_id)
            .add_path("contactId", contact_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_73._unmap(response)

    @cast_models
    def get_users(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        phone_number: str = None,
        email: str = None,
        ewallet_reference_id: str = None,
        page_number: float = None,
        page_size: float = None,
        type_: str = None,
        min_balance: float = None,
        currency: str = None,
        idempotency: str = None,
    ) -> InlineResponse200_97:
        """Retrieve a list of Rapyd Wallets. You can filter the list with query parameters.

        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param phone_number: Phone number of the Rapyd Wallet in E.164 format., defaults to None
        :type phone_number: str, optional
        :param email: Email address of the wallet owner., defaults to None
        :type email: str, optional
        :param ewallet_reference_id: Wallet ID defined by the customer or end user., defaults to None
        :type ewallet_reference_id: str, optional
        :param page_number: Page number to retrieve. If `page_number` is not specified, page 1 is retrieved., defaults to None
        :type page_number: float, optional
        :param page_size: Number of results per page., defaults to None
        :type page_size: float, optional
        :param type_: Type of wallet - company, person, client., defaults to None
        :type type_: str, optional
        :param min_balance: min_balance, defaults to None
        :type min_balance: float, optional
        :param currency: currency, defaults to None
        :type currency: str, optional
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List of eWallets,
        :rtype: InlineResponse200_97
        """

        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(phone_number)
        Validator(str).is_optional().validate(email)
        Validator(str).is_optional().validate(ewallet_reference_id)
        Validator(float).is_optional().validate(page_number)
        Validator(float).is_optional().validate(page_size)
        Validator(str).is_optional().validate(type_)
        Validator(float).is_optional().validate(min_balance)
        Validator(str).is_optional().validate(currency)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/ewallets", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_query("phone_number", phone_number, explode=False)
            .add_query("email", email, explode=False)
            .add_query("ewallet_reference_id", ewallet_reference_id, explode=False)
            .add_query("page_number", page_number, explode=False)
            .add_query("page_size", page_size, explode=False)
            .add_query("type", type_, explode=False)
            .add_query("min_balance", min_balance, explode=False)
            .add_query("currency", currency, explode=False)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_97._unmap(response)

    @cast_models
    def create_user(
        self,
        request_body: V1EwalletsBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_98:
        """Create a Rapyd Wallet.

        :param request_body: The request body.
        :type request_body: V1EwalletsBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Rapyd wallet created.
        :rtype: InlineResponse200_98
        """

        Validator(V1EwalletsBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/ewallets", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_98._unmap(response)

    @cast_models
    def get_user(
        self,
        ewallet_token: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_98:
        """Retrieve the details of a Rapyd Wallet.

        :param ewallet_token: ID of the wallet. String starting with **ewallet_**.
        :type ewallet_token: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Retrieve the details of a Rapyd Wallet.
        :rtype: InlineResponse200_98
        """

        Validator(str).validate(ewallet_token)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/ewallets/{{ewalletToken}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("ewalletToken", ewallet_token)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_98._unmap(response)

    @cast_models
    def updated_user(
        self,
        request_body: EwalletsEwalletTokenBody,
        ewallet_token: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_98:
        """Change or modify a Rapyd Wallet.

        :param request_body: The request body.
        :type request_body: EwalletsEwalletTokenBody
        :param ewallet_token: ID of the wallet. String starting with **ewallet_**.
        :type ewallet_token: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Created Rapyd Wallet,
        :rtype: InlineResponse200_98
        """

        Validator(EwalletsEwalletTokenBody).validate(request_body)
        Validator(str).validate(ewallet_token)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/ewallets/{{ewalletToken}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("ewalletToken", ewallet_token)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_98._unmap(response)

    @cast_models
    def delete_user(
        self,
        ewallet_token: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_66:
        """Delete a Rapyd Wallet. Use this method when the wallet has never been used. This method triggers the Wallet Deleted webhook. This webhook contains the same information as the response.

        :param ewallet_token: ID of the wallet. String starting with **ewallet_**.
        :type ewallet_token: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Wallet was deleted.
        :rtype: InlineResponse200_66
        """

        Validator(str).validate(ewallet_token)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/ewallets/{{ewalletToken}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("ewalletToken", ewallet_token)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_66._unmap(response)

    @cast_models
    def update_ewallet_status(
        self,
        ewallet_token: str,
        status: UpdateEwalletStatusStatus,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_66:
        """Change Wallet Status

        :param ewallet_token: ID of the Rapyd Wallet. String starting with **ewallet_**. Required when phone number is not used.
        :type ewallet_token: str
        :param status: Status of the wallet. One of the following: ACT - Active, DIS - Disabled, CLO - Close.
        :type status: UpdateEwalletStatusStatus
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Operations status
        :rtype: InlineResponse200_66
        """

        Validator(str).validate(ewallet_token)
        Validator(UpdateEwalletStatusStatus).validate(status)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/ewallets/{{ewalletToken}}/statuses/{{status}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("ewalletToken", ewallet_token)
            .add_path("status", status)
            .serialize()
            .set_method("POST")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_66._unmap(response)

    @cast_models
    def set_account_limit(
        self,
        wallet_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        request_body: AccountLimitsBody = None,
        idempotency: str = None,
    ) -> InlineResponse200_99:
        """Set the maximum balance limit or the minimum balance threshold for an existing wallet account.

        :param request_body: The request body., defaults to None
        :type request_body: AccountLimitsBody, optional
        :param wallet_id: ID of the Rapyd Wallet that this contact is associated with. String starting with **ewallet_**.
        :type wallet_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List Contacts for Rapyd Wallet Accounts.
        :rtype: InlineResponse200_99
        """

        Validator(AccountLimitsBody).is_optional().validate(request_body)
        Validator(str).validate(wallet_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/ewallets/{{walletId}}/account/limits",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("walletId", wallet_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_99._unmap(response)

    @cast_models
    def remove_account_limit(
        self,
        wallet_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_100:
        """Delete a limit on a wallet account.

        :param wallet_id: ID of the Rapyd Wallet that this contact is associated with. String starting with **ewallet_**.
        :type wallet_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Coupon was deleted.
        :rtype: InlineResponse200_100
        """

        Validator(str).validate(wallet_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/ewallets/{{walletId}}/account/limits",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("walletId", wallet_id)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_100._unmap(response)

    @cast_models
    def get_user_accounts(
        self,
        wallet_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_99:
        """Retrieve details of the balances in a Rapyd Wallet.

        :param wallet_id: ID of the Rapyd Wallet that this contact is associated with. String starting with **ewallet_**.
        :type wallet_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List accounts related to the Rapyd Wallet,
        :rtype: InlineResponse200_99
        """

        Validator(str).validate(wallet_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/ewallets/{{walletId}}/accounts",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("walletId", wallet_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_99._unmap(response)

    @cast_models
    def get_user_transactions(
        self,
        wallet_id: str,
        balance: float,
        page_size: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        currency: str = None,
        end_date: str = None,
        ending_before: str = None,
        page_number: str = None,
        start_date: str = None,
        starting_after: str = None,
        type_: str = None,
        idempotency: str = None,
    ) -> InlineResponse200_101:
        """Retrieve a list of all transactions related to a wallet.

        :param wallet_id: ID of the wallet. String starting with **ewallet_**.
        :type wallet_id: str
        :param balance: The updated wallet balance after successful completion of the transaction.
        :type balance: float
        :param page_size: Number of results per page.
        :type page_size: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param currency: Three-letter ISO 4217 code for the currency of the transactions. Uppercase., defaults to None
        :type currency: str, optional
        :param end_date: Timestamp of the last transaction or later, in Unix time., defaults to None
        :type end_date: str, optional
        :param ending_before: The ID of the wallet transaction created after the last wallet transaction you want to retrieve. String starting with **wt_**., defaults to None
        :type ending_before: str, optional
        :param page_number: Page number to retrieve., defaults to None
        :type page_number: str, optional
        :param start_date: Timestamp of the first transaction or earlier, in Unix time., defaults to None
        :type start_date: str, optional
        :param starting_after: The ID of the wallet transaction created before the first wallet transaction you want to retrieve. String starting with **wt_**., defaults to None
        :type starting_after: str, optional
        :param type_: Type of transaction., defaults to None
        :type type_: str, optional
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List of all transactions related to a wallet.
        :rtype: InlineResponse200_101
        """

        Validator(str).validate(wallet_id)
        Validator(float).validate(balance)
        Validator(str).validate(page_size)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(currency)
        Validator(str).is_optional().validate(end_date)
        Validator(str).is_optional().validate(ending_before)
        Validator(str).is_optional().validate(page_number)
        Validator(str).is_optional().validate(start_date)
        Validator(str).is_optional().validate(starting_after)
        Validator(str).is_optional().validate(type_)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/ewallets/{{walletId}}/transactions",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("walletId", wallet_id)
            .add_query("balance", balance, explode=False)
            .add_query("currency", currency, explode=False)
            .add_query("end_date", end_date, explode=False)
            .add_query("ending_before", ending_before, explode=False)
            .add_query("page_number", page_number, explode=False)
            .add_query("page_size", page_size, explode=False)
            .add_query("start_date", start_date, explode=False)
            .add_query("starting_after", starting_after, explode=False)
            .add_query("type", type_, explode=False)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_101._unmap(response)

    @cast_models
    def get_user_transaction_details(
        self,
        wallet_id: str,
        transaction_id: float,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_102:
        """Retrieve the details of a wallet transaction.

        :param wallet_id: ID of the wallet. String starting with **ewallet_**.
        :type wallet_id: str
        :param transaction_id: ID of the transaction, from the response to List Wallet Transactions. String starting with wt_.
        :type transaction_id: float
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Retrieve the details of a wallet transaction.
        :rtype: InlineResponse200_102
        """

        Validator(str).validate(wallet_id)
        Validator(float).validate(transaction_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/ewallets/{{walletId}}/transactions/{{transactionId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("walletId", wallet_id)
            .add_path("transactionId", transaction_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_102._unmap(response)

    @cast_models
    def virtual_accounts_by_rapyd_wallet(
        self,
        ewallet: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_103:
        """List the virtual accounts that are attached to a Rapyd Wallet.

        :param ewallet: ID of the Rapyd Wallet that the virtual accounts were issued to. String starting with **ewallet_**.
        :type ewallet: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Returns capabilities of virtual accounts.
        :rtype: InlineResponse200_103
        """

        Validator(str).validate(ewallet)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/ewallets/{{ewallet}}/virtual_accounts",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("ewallet", ewallet)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_103._unmap(response)



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .payment_method_type import PaymentMethodType


@JsonMap({})
class CustomerIdPaymentMethodsBody(BaseModel):
    """CustomerIdPaymentMethodsBody

    :param data: A type of payment method that a customer can use for making payments., defaults to None
    :type data: PaymentMethodType, optional
    """

    def __init__(self, data: PaymentMethodType = None):
        """CustomerIdPaymentMethodsBody

        :param data: A type of payment method that a customer can use for making payments., defaults to None
        :type data: PaymentMethodType, optional
        """
        self.data = self._define_object(data, PaymentMethodType)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({"type_": "type"})
class BinDetails(BaseModel):
    """Bank Identification Number (BIN) details. Read-only. Object containing the following fields - * bin_number - BIN number * country - The two-letter ISO 3166-1 ALPHA-2 code for the country. Uppercase. * funding - Type of card funding. One of the following [credit, debit, prepaid, unknown] * bank - Name of the issuing bank. Relevant to cards

    :param brand: brand, defaults to None
    :type brand: str, optional
    :param bin_number: bin_number, defaults to None
    :type bin_number: str, optional
    :param type_: type_, defaults to None
    :type type_: str, optional
    :param issuer: issuer, defaults to None
    :type issuer: str, optional
    :param country: country, defaults to None
    :type country: str, optional
    :param level: level, defaults to None
    :type level: str, optional
    """

    def __init__(
        self,
        brand: str = None,
        bin_number: str = None,
        type_: str = None,
        issuer: str = None,
        country: str = None,
        level: str = None,
    ):
        """Bank Identification Number (BIN) details. Read-only. Object containing the following fields - * bin_number - BIN number * country - The two-letter ISO 3166-1 ALPHA-2 code for the country. Uppercase. * funding - Type of card funding. One of the following [credit, debit, prepaid, unknown] * bank - Name of the issuing bank. Relevant to cards

        :param brand: brand, defaults to None
        :type brand: str, optional
        :param bin_number: bin_number, defaults to None
        :type bin_number: str, optional
        :param type_: type_, defaults to None
        :type type_: str, optional
        :param issuer: issuer, defaults to None
        :type issuer: str, optional
        :param country: country, defaults to None
        :type country: str, optional
        :param level: level, defaults to None
        :type level: str, optional
        """
        self.brand = self._define_str("brand", brand, nullable=True)
        self.bin_number = self._define_str("bin_number", bin_number, nullable=True)
        self.type_ = self._define_str("type_", type_, nullable=True)
        self.issuer = self._define_str("issuer", issuer, nullable=True)
        self.country = self._define_str(
            "country",
            country,
            nullable=True,
            pattern="Name of the country. Two-letter ISO 3166-1 alpha-2 code.",
        )
        self.level = self._define_str("level", level, nullable=True)



from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .v1cnltermination_query_queried_merchant_principals_address import (
    V1cnlterminationQueryQueriedMerchantPrincipalsAddress,
)


class InlineResponse200_119DataMatchedMerchantBusinessCategory(Enum):
    """An enumeration representing different categories.

    :cvar MERCHANT: "Merchant"
    :vartype MERCHANT: str
    :cvar PAYMENTFACILITATOR: "Payment Facilitator"
    :vartype PAYMENTFACILITATOR: str
    :cvar INDEPENDENTSALESORGANIZATION: "Independent Sales Organization"
    :vartype INDEPENDENTSALESORGANIZATION: str
    :cvar MARKETPLACE: "Marketplace"
    :vartype MARKETPLACE: str
    :cvar STAGEDDIGITALWALLETOPERATOR: "Staged Digital Wallet Operator"
    :vartype STAGEDDIGITALWALLETOPERATOR: str
    :cvar SPONSOREDMERCHANT: "Sponsored Merchant"
    :vartype SPONSOREDMERCHANT: str
    """

    MERCHANT = "Merchant"
    PAYMENTFACILITATOR = "Payment Facilitator"
    INDEPENDENTSALESORGANIZATION = "Independent Sales Organization"
    MARKETPLACE = "Marketplace"
    STAGEDDIGITALWALLETOPERATOR = "Staged Digital Wallet Operator"
    SPONSOREDMERCHANT = "Sponsored Merchant"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                InlineResponse200_119DataMatchedMerchantBusinessCategory._member_map_.values(),
            )
        )


@JsonMap({})
class InlineResponse200_119DataMatchedMerchant(BaseModel):
    """Data about the merchant on file in the database.

    :param business_category: The category of the merchant's business., defaults to None
    :type business_category: InlineResponse200_119DataMatchedMerchantBusinessCategory, optional
    :param dba_name: The "doing business as" name of the merchant.\<BR\>String that starts and ends with any combination of characters, whitespace, a specified range of Latin letters with diacritics, and a specific set of special characters.\<BR\>Length: 4-60, defaults to None
    :type dba_name: str, optional
    :param legal_name: The registered legal name of the merchant., defaults to None
    :type legal_name: str, optional
    :param address: Details of the owner's business address., defaults to None
    :type address: V1cnlterminationQueryQueriedMerchantPrincipalsAddress, optional
    :param phone_numbers: Business phone numbers of the merchant.\<BR\<Array of strings. Maximum - 2. Each string is a phone number in the format +[country code]/[phone number], where the forward slash (/) represents a separator between the country code and the telephone number.\<BR\>The plus sign before the country code is optional. The country code may only contain digits.\<BR\>The slash is required and must follow the country code.\<BR\>The telephone number may include blank spaces and hyphens (-).\<BR\>For example, in **+1/555 555-5555**, the country code is 1 and the telephone number is 5555555555.\<BR\>Length of each phone number: 8-15., defaults to None
    :type phone_numbers: dict, optional
    :param mcc: Merchant category codes of the merchant's business.\<BR\>Array of strings. Numbers or US letters. Maximum - 5 items. \<BR\>Length of each string: 4., defaults to None
    :type mcc: dict, optional
    :param url: Web addresses associated with the merchant.\<BR\>Array of strings. Numbers or US letters. Maximum - 3 items. For example: ["https://fourstarmarket.com", "https://fourstarmarket.net"]\<BR\>Length: Maximum - 40, defaults to None
    :type url: dict, optional
    """

    def __init__(
        self,
        business_category: InlineResponse200_119DataMatchedMerchantBusinessCategory = None,
        dba_name: str = None,
        legal_name: str = None,
        address: V1cnlterminationQueryQueriedMerchantPrincipalsAddress = None,
        phone_numbers: dict = None,
        mcc: dict = None,
        url: dict = None,
    ):
        """Data about the merchant on file in the database.

        :param business_category: The category of the merchant's business., defaults to None
        :type business_category: InlineResponse200_119DataMatchedMerchantBusinessCategory, optional
        :param dba_name: The "doing business as" name of the merchant.\<BR\>String that starts and ends with any combination of characters, whitespace, a specified range of Latin letters with diacritics, and a specific set of special characters.\<BR\>Length: 4-60, defaults to None
        :type dba_name: str, optional
        :param legal_name: The registered legal name of the merchant., defaults to None
        :type legal_name: str, optional
        :param address: Details of the owner's business address., defaults to None
        :type address: V1cnlterminationQueryQueriedMerchantPrincipalsAddress, optional
        :param phone_numbers: Business phone numbers of the merchant.\<BR\<Array of strings. Maximum - 2. Each string is a phone number in the format +[country code]/[phone number], where the forward slash (/) represents a separator between the country code and the telephone number.\<BR\>The plus sign before the country code is optional. The country code may only contain digits.\<BR\>The slash is required and must follow the country code.\<BR\>The telephone number may include blank spaces and hyphens (-).\<BR\>For example, in **+1/555 555-5555**, the country code is 1 and the telephone number is 5555555555.\<BR\>Length of each phone number: 8-15., defaults to None
        :type phone_numbers: dict, optional
        :param mcc: Merchant category codes of the merchant's business.\<BR\>Array of strings. Numbers or US letters. Maximum - 5 items. \<BR\>Length of each string: 4., defaults to None
        :type mcc: dict, optional
        :param url: Web addresses associated with the merchant.\<BR\>Array of strings. Numbers or US letters. Maximum - 3 items. For example: ["https://fourstarmarket.com", "https://fourstarmarket.net"]\<BR\>Length: Maximum - 40, defaults to None
        :type url: dict, optional
        """
        self.business_category = (
            self._enum_matching(
                business_category,
                InlineResponse200_119DataMatchedMerchantBusinessCategory.list(),
                "business_category",
            )
            if business_category
            else None
        )
        self.dba_name = self._define_str("dba_name", dba_name, nullable=True)
        self.legal_name = self._define_str("legal_name", legal_name, nullable=True)
        self.address = self._define_object(
            address, V1cnlterminationQueryQueriedMerchantPrincipalsAddress
        )
        self.phone_numbers = phone_numbers
        self.mcc = mcc
        self.url = url



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class InlineResponse200_116DataApplicationType(BaseModel):
    """InlineResponse200_116DataApplicationType

    :param application_type: application_type, defaults to None
    :type application_type: str, optional
    :param entity_type: entity_type, defaults to None
    :type entity_type: str, optional
    :param country: country, defaults to None
    :type country: str, optional
    """

    def __init__(
        self, application_type: str = None, entity_type: str = None, country: str = None
    ):
        """InlineResponse200_116DataApplicationType

        :param application_type: application_type, defaults to None
        :type application_type: str, optional
        :param entity_type: entity_type, defaults to None
        :type entity_type: str, optional
        :param country: country, defaults to None
        :type country: str, optional
        """
        self.application_type = self._define_str(
            "application_type", application_type, nullable=True
        )
        self.entity_type = self._define_str("entity_type", entity_type, nullable=True)
        self.country = self._define_str("country", country, nullable=True)



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .escrow_ewallets import EscrowEwallets


@JsonMap({"id_": "id"})
class EscrowResponseEscrowReleasesData(BaseModel):
    """Array of objects that describe individual escrow releases.

    :param amount: The currency of the escrow balance. Three-letter ISO 4217 code., defaults to None
    :type amount: float, optional
    :param created_at: Date and time the escrow release was created, in Unix time., defaults to None
    :type created_at: float, optional
    :param currency: The currency of the escrow balance. Three-letter ISO 4217 code., defaults to None
    :type currency: str, optional
    :param ewallets: Describes the wallets and the releases from escrow., defaults to None
    :type ewallets: EscrowEwallets, optional
    :param id_: ID of the escrow release, a string starting with **er_**., defaults to None
    :type id_: float, optional
    :param proportional_release: Indicates that the wallets were released in the same proportion that was defined in the Create Payment request., defaults to None
    :type proportional_release: bool, optional
    :param trigger: Indicates what triggered the escrow release. One of the following: \<BR\> * **event** - Funds were released due to an API request. \<BR\> **time** - Funds were automatically released at 5:00 p.m. GMT (17:00 GMT) on the day indicated in theCreate Payment request. \<BR\> **cancel_escrow** - Escrow was canceled., defaults to None
    :type trigger: str, optional
    """

    def __init__(
        self,
        amount: float = None,
        created_at: float = None,
        currency: str = None,
        ewallets: EscrowEwallets = None,
        id_: float = None,
        proportional_release: bool = None,
        trigger: str = None,
    ):
        """Array of objects that describe individual escrow releases.

        :param amount: The currency of the escrow balance. Three-letter ISO 4217 code., defaults to None
        :type amount: float, optional
        :param created_at: Date and time the escrow release was created, in Unix time., defaults to None
        :type created_at: float, optional
        :param currency: The currency of the escrow balance. Three-letter ISO 4217 code., defaults to None
        :type currency: str, optional
        :param ewallets: Describes the wallets and the releases from escrow., defaults to None
        :type ewallets: EscrowEwallets, optional
        :param id_: ID of the escrow release, a string starting with **er_**., defaults to None
        :type id_: float, optional
        :param proportional_release: Indicates that the wallets were released in the same proportion that was defined in the Create Payment request., defaults to None
        :type proportional_release: bool, optional
        :param trigger: Indicates what triggered the escrow release. One of the following: \<BR\> * **event** - Funds were released due to an API request. \<BR\> **time** - Funds were automatically released at 5:00 p.m. GMT (17:00 GMT) on the day indicated in theCreate Payment request. \<BR\> **cancel_escrow** - Escrow was canceled., defaults to None
        :type trigger: str, optional
        """
        self.amount = self._define_number("amount", amount, nullable=True)
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.currency = self._define_str("currency", currency, nullable=True)
        self.ewallets = self._define_object(ewallets, EscrowEwallets)
        self.id_ = self._define_number("id_", id_, nullable=True)
        self.proportional_release = proportional_release
        self.trigger = self._define_str("trigger", trigger, nullable=True)



from __future__ import annotations
from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .discount import Discount
from .invoice_item_response import InvoiceItemResponse
from .payment import Payment
from .payout import Payout


class BillingReason(Enum):
    """An enumeration representing different categories.

    :cvar SUBSCRIPTIONCYCLE: "subscription_cycle"
    :vartype SUBSCRIPTIONCYCLE: str
    :cvar SUBSCRIPTIONCREATE: "subscription_create"
    :vartype SUBSCRIPTIONCREATE: str
    :cvar MANUAL: "manual"
    :vartype MANUAL: str
    :cvar UPCOMING: "upcoming"
    :vartype UPCOMING: str
    """

    SUBSCRIPTIONCYCLE = "subscription_cycle"
    SUBSCRIPTIONCREATE = "subscription_create"
    MANUAL = "manual"
    UPCOMING = "upcoming"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, BillingReason._member_map_.values()))


class InvoiceResponseStatus(Enum):
    """An enumeration representing different categories.

    :cvar DRAFT: "draft"
    :vartype DRAFT: str
    :cvar DELETED: "deleted"
    :vartype DELETED: str
    :cvar OPEN: "open"
    :vartype OPEN: str
    :cvar PAID: "paid"
    :vartype PAID: str
    :cvar VOID: "void"
    :vartype VOID: str
    :cvar UNCOLLECTIBLE: "uncollectible"
    :vartype UNCOLLECTIBLE: str
    """

    DRAFT = "draft"
    DELETED = "deleted"
    OPEN = "open"
    PAID = "paid"
    VOID = "void"
    UNCOLLECTIBLE = "uncollectible"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, InvoiceResponseStatus._member_map_.values()))


class InvoiceResponseType(Enum):
    """An enumeration representing different categories.

    :cvar PAYMENT: "payment"
    :vartype PAYMENT: str
    :cvar PAYOUT: "payout"
    :vartype PAYOUT: str
    """

    PAYMENT = "payment"
    PAYOUT = "payout"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, InvoiceResponseType._member_map_.values()))


@JsonMap({"id_": "id", "type_": "type"})
class InvoiceResponse(BaseModel):
    """Invoice

    :param id_: id_, defaults to None
    :type id_: str, optional
    :param attempt_count: attempt_count, defaults to None
    :type attempt_count: float, optional
    :param attempted: Number of automatic attempts that Rapyd made to retry paying this invoice if it failed., defaults to None
    :type attempted: float, optional
    :param automatic_attempt_count: Number of automatic attempts that Rapyd made to retry paying this invoice if it failed., defaults to None
    :type automatic_attempt_count: float, optional
    :param billing: Determines the method of collection, pay_automatically or send_invoice. Default is pay_automatically, defaults to None
    :type billing: str, optional
    :param billing_reason: The reason for billing, defaults to None
    :type billing_reason: BillingReason, optional
    :param created_at: The time the invoice was created, in Unix time. Response only., defaults to None
    :type created_at: float, optional
    :param discount: Describes the fields relating to discounts in REST messages and webhooks for customer profiles and subscriptions Contains information about the coupon that applies to the customer. Read-only field. Adding a discount is a 2-step process: \<BR\> 1. Create Coupon, which returns a coupon ID. \<BR\>2. Add the coupon ID to the coupon field of the customer with Create Customer or Update Customer., defaults to None
    :type discount: Discount, optional
    :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
    :type currency: str, optional
    :param customer: ID of the customer. String starting with cus_., defaults to None
    :type customer: str, optional
    :param days_until_due: The number of days until the due date., defaults to None
    :type days_until_due: float, optional
    :param description: Description of the invoice., defaults to None
    :type description: str, optional
    :param discount_amount: The amount of the discount that was applied to this invoice., defaults to None
    :type discount_amount: float, optional
    :param due_date: The date payment is due on this invoice. This value is calculated from the date the invoice is created, plus the number of days specified in the `days_until_due` field. Format is in Unix time., defaults to None
    :type due_date: str, optional
    :param lines: lines, defaults to None
    :type lines: List[InvoiceItemResponse], optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param payment_fields: Object containing additional payment_options fields., defaults to None
    :type payment_fields: dict, optional
    :param payment_method: Payment method for the invoice., defaults to None
    :type payment_method: str, optional
    :param statement_descriptor: Description of the invoice for the customer's credit card statement. Limited to 22 characters., defaults to None
    :type statement_descriptor: str, optional
    :param subscription: Collects money from a payment method and deposits it into one or more Rapyd Wallets, defaults to None
    :type subscription: Payment, optional
    :param tax_percent: The tax rate, defined as a percentage., defaults to None
    :type tax_percent: float, optional
    :param payment: Collects money from a payment method and deposits it into one or more Rapyd Wallets, defaults to None
    :type payment: Payment, optional
    :param payout: payout, defaults to None
    :type payout: Payout, optional
    :param payout_fields: Reserved, defaults to None
    :type payout_fields: dict, optional
    :param period_end: Last date in the period covered by the invoice, in Unix time. Response only., defaults to None
    :type period_end: float, optional
    :param period_start: First date in the period covered by the invoice, in Unix time. Response only., defaults to None
    :type period_start: float, optional
    :param next_payment_attempt: The time when the next payment attempt will be made, in Unix time., defaults to None
    :type next_payment_attempt: float, optional
    :param number: number, defaults to None
    :type number: str, optional
    :param status: An invoice moves through a series of statuses. See status table below., defaults to None
    :type status: InvoiceResponseStatus, optional
    :param subtotal: Total of all line items before discount and before tax. Decimal. Response only., defaults to None
    :type subtotal: float, optional
    :param total: Total after discount and tax. Decimal., defaults to None
    :type total: float, optional
    :param type_: type_, defaults to None
    :type type_: InvoiceResponseType, optional
    """

    def __init__(
        self,
        id_: str = None,
        attempt_count: float = None,
        attempted: float = None,
        automatic_attempt_count: float = None,
        billing: str = None,
        billing_reason: BillingReason = None,
        created_at: float = None,
        discount: Discount = None,
        currency: str = None,
        customer: str = None,
        days_until_due: float = None,
        description: str = None,
        discount_amount: float = None,
        due_date: str = None,
        lines: List[InvoiceItemResponse] = None,
        metadata: dict = None,
        payment_fields: dict = None,
        payment_method: str = None,
        statement_descriptor: str = None,
        subscription: Payment = None,
        tax_percent: float = None,
        payment: Payment = None,
        payout: Payout = None,
        payout_fields: dict = None,
        period_end: float = None,
        period_start: float = None,
        next_payment_attempt: float = None,
        number: str = None,
        status: InvoiceResponseStatus = None,
        subtotal: float = None,
        total: float = None,
        type_: InvoiceResponseType = None,
    ):
        """Invoice

        :param id_: id_, defaults to None
        :type id_: str, optional
        :param attempt_count: attempt_count, defaults to None
        :type attempt_count: float, optional
        :param attempted: Number of automatic attempts that Rapyd made to retry paying this invoice if it failed., defaults to None
        :type attempted: float, optional
        :param automatic_attempt_count: Number of automatic attempts that Rapyd made to retry paying this invoice if it failed., defaults to None
        :type automatic_attempt_count: float, optional
        :param billing: Determines the method of collection, pay_automatically or send_invoice. Default is pay_automatically, defaults to None
        :type billing: str, optional
        :param billing_reason: The reason for billing, defaults to None
        :type billing_reason: BillingReason, optional
        :param created_at: The time the invoice was created, in Unix time. Response only., defaults to None
        :type created_at: float, optional
        :param discount: Describes the fields relating to discounts in REST messages and webhooks for customer profiles and subscriptions Contains information about the coupon that applies to the customer. Read-only field. Adding a discount is a 2-step process: \<BR\> 1. Create Coupon, which returns a coupon ID. \<BR\>2. Add the coupon ID to the coupon field of the customer with Create Customer or Update Customer., defaults to None
        :type discount: Discount, optional
        :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
        :type currency: str, optional
        :param customer: ID of the customer. String starting with cus_., defaults to None
        :type customer: str, optional
        :param days_until_due: The number of days until the due date., defaults to None
        :type days_until_due: float, optional
        :param description: Description of the invoice., defaults to None
        :type description: str, optional
        :param discount_amount: The amount of the discount that was applied to this invoice., defaults to None
        :type discount_amount: float, optional
        :param due_date: The date payment is due on this invoice. This value is calculated from the date the invoice is created, plus the number of days specified in the `days_until_due` field. Format is in Unix time., defaults to None
        :type due_date: str, optional
        :param lines: lines, defaults to None
        :type lines: List[InvoiceItemResponse], optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param payment_fields: Object containing additional payment_options fields., defaults to None
        :type payment_fields: dict, optional
        :param payment_method: Payment method for the invoice., defaults to None
        :type payment_method: str, optional
        :param statement_descriptor: Description of the invoice for the customer's credit card statement. Limited to 22 characters., defaults to None
        :type statement_descriptor: str, optional
        :param subscription: Collects money from a payment method and deposits it into one or more Rapyd Wallets, defaults to None
        :type subscription: Payment, optional
        :param tax_percent: The tax rate, defined as a percentage., defaults to None
        :type tax_percent: float, optional
        :param payment: Collects money from a payment method and deposits it into one or more Rapyd Wallets, defaults to None
        :type payment: Payment, optional
        :param payout: payout, defaults to None
        :type payout: Payout, optional
        :param payout_fields: Reserved, defaults to None
        :type payout_fields: dict, optional
        :param period_end: Last date in the period covered by the invoice, in Unix time. Response only., defaults to None
        :type period_end: float, optional
        :param period_start: First date in the period covered by the invoice, in Unix time. Response only., defaults to None
        :type period_start: float, optional
        :param next_payment_attempt: The time when the next payment attempt will be made, in Unix time., defaults to None
        :type next_payment_attempt: float, optional
        :param number: number, defaults to None
        :type number: str, optional
        :param status: An invoice moves through a series of statuses. See status table below., defaults to None
        :type status: InvoiceResponseStatus, optional
        :param subtotal: Total of all line items before discount and before tax. Decimal. Response only., defaults to None
        :type subtotal: float, optional
        :param total: Total after discount and tax. Decimal., defaults to None
        :type total: float, optional
        :param type_: type_, defaults to None
        :type type_: InvoiceResponseType, optional
        """
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.attempt_count = self._define_number(
            "attempt_count", attempt_count, nullable=True
        )
        self.attempted = self._define_number("attempted", attempted, nullable=True)
        self.automatic_attempt_count = self._define_number(
            "automatic_attempt_count", automatic_attempt_count, nullable=True
        )
        self.billing = self._define_str("billing", billing, nullable=True)
        self.billing_reason = (
            self._enum_matching(billing_reason, BillingReason.list(), "billing_reason")
            if billing_reason
            else None
        )
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.discount = self._define_object(discount, Discount)
        self.currency = self._define_str(
            "currency",
            currency,
            nullable=True,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.customer = self._define_str("customer", customer, nullable=True)
        self.days_until_due = self._define_number(
            "days_until_due", days_until_due, nullable=True
        )
        self.description = self._define_str("description", description, nullable=True)
        self.discount_amount = self._define_number(
            "discount_amount", discount_amount, nullable=True
        )
        self.due_date = self._define_str("due_date", due_date, nullable=True)
        self.lines = self._define_list(lines, InvoiceItemResponse)
        self.metadata = metadata
        self.payment_fields = payment_fields
        self.payment_method = self._define_str(
            "payment_method", payment_method, nullable=True
        )
        self.statement_descriptor = self._define_str(
            "statement_descriptor", statement_descriptor, nullable=True
        )
        self.subscription = self._define_object(subscription, Payment)
        self.tax_percent = self._define_number(
            "tax_percent", tax_percent, nullable=True
        )
        self.payment = self._define_object(payment, Payment)
        self.payout = self._define_object(payout, Payout)
        self.payout_fields = payout_fields
        self.period_end = self._define_number("period_end", period_end, nullable=True)
        self.period_start = self._define_number(
            "period_start", period_start, nullable=True
        )
        self.next_payment_attempt = self._define_number(
            "next_payment_attempt", next_payment_attempt, nullable=True
        )
        self.number = self._define_str("number", number, nullable=True)
        self.status = (
            self._enum_matching(status, InvoiceResponseStatus.list(), "status")
            if status
            else None
        )
        self.subtotal = self._define_number("subtotal", subtotal, nullable=True)
        self.total = self._define_number("total", total, nullable=True)
        self.type_ = (
            self._enum_matching(type_, InvoiceResponseType.list(), "type_")
            if type_
            else None
        )



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .v1ewallets_contact import V1ewalletsContact


@JsonMap({"type_": "type"})
class V1EwalletsBody(BaseModel):
    """V1EwalletsBody

    :param contact: contact
    :type contact: V1ewalletsContact
    :param ewallet_reference_id: Rapyd Wallet ID defined by the customer or end user. Must be unique., defaults to None
    :type ewallet_reference_id: str, optional
    :param first_name: First name of the Rapyd Wallet owner., defaults to None
    :type first_name: str, optional
    :param last_name: Last name of the Rapyd Wallet owner., defaults to None
    :type last_name: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param type_: Type of wallet, company or person. Default is person., defaults to None
    :type type_: str, optional
    """

    def __init__(
        self,
        contact: V1ewalletsContact,
        ewallet_reference_id: str = None,
        first_name: str = None,
        last_name: str = None,
        metadata: dict = None,
        type_: str = None,
    ):
        """V1EwalletsBody

        :param contact: contact
        :type contact: V1ewalletsContact
        :param ewallet_reference_id: Rapyd Wallet ID defined by the customer or end user. Must be unique., defaults to None
        :type ewallet_reference_id: str, optional
        :param first_name: First name of the Rapyd Wallet owner., defaults to None
        :type first_name: str, optional
        :param last_name: Last name of the Rapyd Wallet owner., defaults to None
        :type last_name: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param type_: Type of wallet, company or person. Default is person., defaults to None
        :type type_: str, optional
        """
        self.contact = self._define_object(contact, V1ewalletsContact)
        self.ewallet_reference_id = self._define_str(
            "ewallet_reference_id", ewallet_reference_id, nullable=True
        )
        self.first_name = self._define_str("first_name", first_name, nullable=True)
        self.last_name = self._define_str("last_name", last_name, nullable=True)
        self.metadata = metadata
        self.type_ = self._define_str("type_", type_, nullable=True)



from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


class PayoutRequiredFieldsType(Enum):
    """An enumeration representing different categories.

    :cvar BOOLEAN: "boolean"
    :vartype BOOLEAN: str
    :cvar NUMBER: "number"
    :vartype NUMBER: str
    :cvar STRING: "string"
    :vartype STRING: str
    """

    BOOLEAN = "boolean"
    NUMBER = "number"
    STRING = "string"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, PayoutRequiredFieldsType._member_map_.values())
        )


@JsonMap({"type_": "type"})
class PayoutRequiredFields(BaseModel):
    """PayoutRequiredFields

    :param allowed_values: List out the allowed values, defaults to None
    :type allowed_values: str, optional
    :param description: Description of the field, defaults to None
    :type description: str, optional
    :param is_required: Indicates whether the field is always required for using the payout method, defaults to None
    :type is_required: bool, optional
    :param name: Name of the field, defaults to None
    :type name: str, optional
    :param regex: A regular expression that defines the format when type is string. Note: Rapyd uses a unique variant of the regex standard. See note in           "https://docs.rapyd.net/en/get-payment-method-required-fields.html" , defaults to None
    :type regex: str, optional
    :param transfer_type: Indicates transfer type of the payout, defaults to None
    :type transfer_type: str, optional
    :param type_: Indicates datatype of the field, defaults to None
    :type type_: PayoutRequiredFieldsType, optional
    """

    def __init__(
        self,
        allowed_values: str = None,
        description: str = None,
        is_required: bool = None,
        name: str = None,
        regex: str = None,
        transfer_type: str = None,
        type_: PayoutRequiredFieldsType = None,
    ):
        """PayoutRequiredFields

        :param allowed_values: List out the allowed values, defaults to None
        :type allowed_values: str, optional
        :param description: Description of the field, defaults to None
        :type description: str, optional
        :param is_required: Indicates whether the field is always required for using the payout method, defaults to None
        :type is_required: bool, optional
        :param name: Name of the field, defaults to None
        :type name: str, optional
        :param regex: A regular expression that defines the format when type is string. Note: Rapyd uses a unique variant of the regex standard. See note in           "https://docs.rapyd.net/en/get-payment-method-required-fields.html" , defaults to None
        :type regex: str, optional
        :param transfer_type: Indicates transfer type of the payout, defaults to None
        :type transfer_type: str, optional
        :param type_: Indicates datatype of the field, defaults to None
        :type type_: PayoutRequiredFieldsType, optional
        """
        self.allowed_values = self._define_str(
            "allowed_values", allowed_values, nullable=True
        )
        self.description = self._define_str("description", description, nullable=True)
        self.is_required = is_required
        self.name = self._define_str("name", name, nullable=True)
        self.regex = self._define_str("regex", regex, nullable=True)
        self.transfer_type = self._define_str(
            "transfer_type", transfer_type, nullable=True
        )
        self.type_ = (
            self._enum_matching(type_, PayoutRequiredFieldsType.list(), "type_")
            if type_
            else None
        )



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .inline_response_200_93_data import InlineResponse200_93Data
from .status import Status


@JsonMap({})
class InlineResponse200_93(BaseModel):
    """InlineResponse200_93

    :param data: data, defaults to None
    :type data: InlineResponse200_93Data, optional
    :param status: status, defaults to None
    :type status: Status, optional
    """

    def __init__(self, data: InlineResponse200_93Data = None, status: Status = None):
        """InlineResponse200_93

        :param data: data, defaults to None
        :type data: InlineResponse200_93Data, optional
        :param status: status, defaults to None
        :type status: Status, optional
        """
        self.data = self._define_object(data, InlineResponse200_93Data)
        self.status = self._define_object(status, Status)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class ApplicationsHostedBody(BaseModel):
    """ApplicationsHostedBody

    :param application_type: Code for the type of application., defaults to None
    :type application_type: str, optional
    :param country: The country where the company is domiciled. Two-letter ISO 3166-1 ALPHA-2 code. The hosted application displays the country as the default setting.
    :type country: str
    :param rapyd_entity_token: The ID of the Rapyd wallet of the company. String starting with ewallet_.
    :type rapyd_entity_token: str
    :param phone_number: The phone number of the applicant. This is the phone number where an authentication code is sent. Must have a leading plus sign (+).
    :type phone_number: str
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param client_reference_id: ID defined by the client. Limited to 255 characters., defaults to None
    :type client_reference_id: str, optional
    :param cancel_url: URL where the customer is redirected after pressing Back to Website to exit the hosted page. This URL overrides the merchant_website URL. Does not support localhost URLs., defaults to None
    :type cancel_url: str, optional
    :param complete_url: URL where the customer is redirected after pressing Close to exit the hosted page. This URL overrides the merchant_website URL. Does not support localhost URLs., defaults to None
    :type complete_url: str, optional
    """

    def __init__(
        self,
        country: str,
        rapyd_entity_token: str,
        phone_number: str,
        application_type: str = None,
        metadata: dict = None,
        client_reference_id: str = None,
        cancel_url: str = None,
        complete_url: str = None,
    ):
        """ApplicationsHostedBody

        :param application_type: Code for the type of application., defaults to None
        :type application_type: str, optional
        :param country: The country where the company is domiciled. Two-letter ISO 3166-1 ALPHA-2 code. The hosted application displays the country as the default setting.
        :type country: str
        :param rapyd_entity_token: The ID of the Rapyd wallet of the company. String starting with ewallet_.
        :type rapyd_entity_token: str
        :param phone_number: The phone number of the applicant. This is the phone number where an authentication code is sent. Must have a leading plus sign (+).
        :type phone_number: str
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param client_reference_id: ID defined by the client. Limited to 255 characters., defaults to None
        :type client_reference_id: str, optional
        :param cancel_url: URL where the customer is redirected after pressing Back to Website to exit the hosted page. This URL overrides the merchant_website URL. Does not support localhost URLs., defaults to None
        :type cancel_url: str, optional
        :param complete_url: URL where the customer is redirected after pressing Close to exit the hosted page. This URL overrides the merchant_website URL. Does not support localhost URLs., defaults to None
        :type complete_url: str, optional
        """
        self.application_type = self._define_str(
            "application_type", application_type, nullable=True
        )
        self.country = country
        self.rapyd_entity_token = rapyd_entity_token
        self.phone_number = phone_number
        self.metadata = metadata
        self.client_reference_id = self._define_str(
            "client_reference_id", client_reference_id, nullable=True
        )
        self.cancel_url = self._define_str("cancel_url", cancel_url, nullable=True)
        self.complete_url = self._define_str(
            "complete_url", complete_url, nullable=True
        )



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class V1skusskuIdPackageDimensions(BaseModel):
    """Physical attributes of the SKU item. Object containing the following fields - height, length, weight, width

    :param length: length, defaults to None
    :type length: float, optional
    :param height: height, defaults to None
    :type height: float, optional
    :param weight: weight, defaults to None
    :type weight: float, optional
    :param width: width, defaults to None
    :type width: float, optional
    """

    def __init__(
        self,
        length: float = None,
        height: float = None,
        weight: float = None,
        width: float = None,
    ):
        """Physical attributes of the SKU item. Object containing the following fields - height, length, weight, width

        :param length: length, defaults to None
        :type length: float, optional
        :param height: height, defaults to None
        :type height: float, optional
        :param weight: weight, defaults to None
        :type weight: float, optional
        :param width: width, defaults to None
        :type width: float, optional
        """
        self.length = self._define_number("length", length, nullable=True)
        self.height = self._define_number("height", height, nullable=True)
        self.weight = self._define_number("weight", weight, nullable=True)
        self.width = self._define_number("width", width, nullable=True)



from __future__ import annotations
from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .hosted_beneficiary_token_response_beneficiary_optional_fields import (
    HostedBeneficiaryTokenResponseBeneficiaryOptionalFields,
)
from .hosted_beneficiary_token_response_merchant_customer_support import (
    HostedBeneficiaryTokenResponseMerchantCustomerSupport,
)


class HostedBeneficiaryTokenResponseBeneficiaryEntityType(Enum):
    """An enumeration representing different categories.

    :cvar COMPANY: "company"
    :vartype COMPANY: str
    :cvar INDIVIDUAL: "individual"
    :vartype INDIVIDUAL: str
    """

    COMPANY = "company"
    INDIVIDUAL = "individual"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                HostedBeneficiaryTokenResponseBeneficiaryEntityType._member_map_.values(),
            )
        )


class HostedBeneficiaryTokenResponseBeneficiaryExtendedFields(Enum):
    """An enumeration representing different categories.

    :cvar TRUE: "true"
    :vartype TRUE: str
    :cvar FALSE: "false"
    :vartype FALSE: str
    """

    TRUE = "true"
    FALSE = "false"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                HostedBeneficiaryTokenResponseBeneficiaryExtendedFields._member_map_.values(),
            )
        )


class HostedBeneficiaryTokenResponseCategory(Enum):
    """An enumeration representing different categories.

    :cvar BANK: "bank"
    :vartype BANK: str
    :cvar CARD: "card"
    :vartype CARD: str
    """

    BANK = "bank"
    CARD = "card"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                HostedBeneficiaryTokenResponseCategory._member_map_.values(),
            )
        )


class HostedBeneficiaryTokenResponseEntityType(Enum):
    """An enumeration representing different categories.

    :cvar COMPANY: "company"
    :vartype COMPANY: str
    :cvar INDIVIDUAL: "individual"
    :vartype INDIVIDUAL: str
    """

    COMPANY = "company"
    INDIVIDUAL = "individual"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                HostedBeneficiaryTokenResponseEntityType._member_map_.values(),
            )
        )


class HostedBeneficiaryTokenResponseSenderEntityType(Enum):
    """An enumeration representing different categories.

    :cvar COMPANY: "company"
    :vartype COMPANY: str
    :cvar INDIVIDUAL: "individual"
    :vartype INDIVIDUAL: str
    """

    COMPANY = "company"
    INDIVIDUAL = "individual"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                HostedBeneficiaryTokenResponseSenderEntityType._member_map_.values(),
            )
        )


class HostedBeneficiaryTokenResponseStatus(Enum):
    """An enumeration representing different categories.

    :cvar NEW: "NEW"
    :vartype NEW: str
    :cvar DON: "DON"
    :vartype DON: str
    :cvar EXP: "EXP"
    :vartype EXP: str
    """

    NEW = "NEW"
    DON = "DON"
    EXP = "EXP"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                HostedBeneficiaryTokenResponseStatus._member_map_.values(),
            )
        )


@JsonMap({"id_": "id"})
class HostedBeneficiaryTokenResponse(BaseModel):
    """HostedBeneficiaryTokenResponse

    :param beneficiary_country: The two-letter ISO 3166-1 ALPHA-2 code for the country of the beneficiary’s bank account. The two-letter prefix of the payout method type matches the beneficiary country code., defaults to None
    :type beneficiary_country: str, optional
    :param beneficiary_currency: Currency accepted by the beneficiary’s bank account. Three-letter ISO 4217 code., defaults to None
    :type beneficiary_currency: str, optional
    :param beneficiary_entity_type: Type of entity for the beneficiary. One of the following:, defaults to None
    :type beneficiary_entity_type: HostedBeneficiaryTokenResponseBeneficiaryEntityType, optional
    :param beneficiary_extended_fields: When the value is **true**, the hosted tokenization page displays additional required fields.\<BR\> * Additional required fields when `beneficiary_entity_type` is **individual**: `address`, `city`, `country`, `date_of_birth`, `first_name`, `gender`, `identification_type`, `identification_value`, `last_name`, `nationality`. \<BR\> * Additional required fields when `beneficiary_entity_type` is **company**: `address`, `city`, `company_name`, `country`, `country_of_incorporation`, `date_of_incorporation`, `identification_type`, `identification_value`.\<BR\> For more information on the additional required fields, see 'Create Extended Beneficiary'.\<BR\> **Note**: Currently only available in the sandbox., defaults to None
    :type beneficiary_extended_fields: HostedBeneficiaryTokenResponseBeneficiaryExtendedFields, optional
    :param beneficiary_id: ID of the beneficiary. String starting with **beneficiary_**., defaults to None
    :type beneficiary_id: str, optional
    :param beneficiary_optional_fields: Additional information about the beneficiary., defaults to None
    :type beneficiary_optional_fields: List[HostedBeneficiaryTokenResponseBeneficiaryOptionalFields], optional
    :param beneficiary_validated: Indicates whether the beneficiary has been validated., defaults to None
    :type beneficiary_validated: bool, optional
    :param cancel_url: URL where the customer is redirected after pressing **Back to Website** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
    :type cancel_url: str, optional
    :param category: The category of payout method. One of the following: \<BR\> * **bank**\<BR\> * **card** - US only., defaults to None
    :type category: HostedBeneficiaryTokenResponseCategory, optional
    :param complete_url: URL where the customer is redirected after pressing **Close** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
    :type complete_url: str, optional
    :param country: The two-letter ISO 3166-1 ALPHA-2 code for the beneficiary’s country. The two-letter prefix of the payout method type must match the beneficiary country code., defaults to None
    :type country: str, optional
    :param currency: Currency accepted by the beneficiary’s bank account. Three-letter ISO 4217 code., defaults to None
    :type currency: str, optional
    :param entity_type: Type of entity for the beneficiary. One of the following:, defaults to None
    :type entity_type: HostedBeneficiaryTokenResponseEntityType, optional
    :param expiration: The page expiration date in Unix time., defaults to None
    :type expiration: str, optional
    :param id_: ID of the beneficiary tokenization page. String starting with **hp_ben_**.The page expiration date in Unix time., defaults to None
    :type id_: str, optional
    :param language: Determines the default language of the hosted page. For a list of values, see 'List Supported Languages'.\<BR\> * When this parameter is null, the language of the user's browser is used.\<BR\> * If the language of the user's browser cannot be determined, the default language is English., defaults to None
    :type language: str, optional
    :param merchant_alias: Client's name., defaults to None
    :type merchant_alias: str, optional
    :param merchant_color: Color of the call-to-action (CTA) button on the hosted page.\<BR\> To configure this field, use the Client Portal., defaults to None
    :type merchant_color: str, optional
    :param merchant_customer_support: Contains details of the client’s customer support. To configure these fields, use the Client Portal., defaults to None
    :type merchant_customer_support: List[HostedBeneficiaryTokenResponseMerchantCustomerSupport], optional
    :param merchant_logo: URL for the image of the client's logo.\<BR\> To configure this field, use the Client Portal., defaults to None
    :type merchant_logo: str, optional
    :param merchant_reference_id: Identifier defined by the client for reference purposes. Limit: 45 characters., defaults to None
    :type merchant_reference_id: str, optional
    :param merchant_website: The URL where the customer is redirected after exiting the hosted page. \<BR\> Relevant when one or both of the following fields is unset:\<BR\> * `cancel_url` \<BR\> * `complete_url`\<BR\> To configure this field, use the Client Portal., defaults to None
    :type merchant_website: str, optional
    :param page_expiration: End of the time when the customer can use the hosted page, in Unix time. If `page_expiration` is not set, the hosted page expires 14 days after creation. \<BR\> **Range**: 1 minute to 30 days., defaults to None
    :type page_expiration: float, optional
    :param payout_currency: Currency accepted by the beneficiary’s bank account. Three-letter ISO 4217 code., defaults to None
    :type payout_currency: str, optional
    :param payout_method_type: The payout method type selected by the customer. The two-letter prefix must match the beneficiary country code., defaults to None
    :type payout_method_type: str, optional
    :param payout_method_types_exclude: List of payout methods that are excluded from display on the beneficiary tokenization page., defaults to None
    :type payout_method_types_exclude: List[dict], optional
    :param payout_method_types_include: Array of strings. List of payout methods that are displayed on the beneficiary tokenization page., defaults to None
    :type payout_method_types_include: List[dict], optional
    :param redirect_url: URL of the hosted page that is shown to the customer., defaults to None
    :type redirect_url: str, optional
    :param sender_country: The two-letter ISO 3166-1 ALPHA-2 code for the sender’s country., defaults to None
    :type sender_country: str, optional
    :param sender_currency: Currency paid from the sender’s wallet. Three-letter ISO 4217 code., defaults to None
    :type sender_currency: str, optional
    :param sender_entity_type: Type of entity for the sender. One of the following:., defaults to None
    :type sender_entity_type: HostedBeneficiaryTokenResponseSenderEntityType, optional
    :param status: Status of the hosted page. One of the following:\<BR\>* **NEW** - The hosted page was created.\<BR\> * **DON** - Done. The beneficiary details were saved.\<BR\> * **EXP** - The hosted page expired., defaults to None
    :type status: HostedBeneficiaryTokenResponseStatus, optional
    :param timestamp: Timestamp for the request to create the beneficiary tokenization object page, in Unix time., defaults to None
    :type timestamp: str, optional
    :param tokenization_page: ID of the beneficiary tokenization page. String starting with **hp_ben_**., defaults to None
    :type tokenization_page: str, optional
    """

    def __init__(
        self,
        beneficiary_country: str = None,
        beneficiary_currency: str = None,
        beneficiary_entity_type: HostedBeneficiaryTokenResponseBeneficiaryEntityType = None,
        beneficiary_extended_fields: HostedBeneficiaryTokenResponseBeneficiaryExtendedFields = None,
        beneficiary_id: str = None,
        beneficiary_optional_fields: List[
            HostedBeneficiaryTokenResponseBeneficiaryOptionalFields
        ] = None,
        beneficiary_validated: bool = None,
        cancel_url: str = None,
        category: HostedBeneficiaryTokenResponseCategory = None,
        complete_url: str = None,
        country: str = None,
        currency: str = None,
        entity_type: HostedBeneficiaryTokenResponseEntityType = None,
        expiration: str = None,
        id_: str = None,
        language: str = None,
        merchant_alias: str = None,
        merchant_color: str = None,
        merchant_customer_support: List[
            HostedBeneficiaryTokenResponseMerchantCustomerSupport
        ] = None,
        merchant_logo: str = None,
        merchant_reference_id: str = None,
        merchant_website: str = None,
        page_expiration: float = None,
        payout_currency: str = None,
        payout_method_type: str = None,
        payout_method_types_exclude: List[dict] = None,
        payout_method_types_include: List[dict] = None,
        redirect_url: str = None,
        sender_country: str = None,
        sender_currency: str = None,
        sender_entity_type: HostedBeneficiaryTokenResponseSenderEntityType = None,
        status: HostedBeneficiaryTokenResponseStatus = None,
        timestamp: str = None,
        tokenization_page: str = None,
    ):
        """HostedBeneficiaryTokenResponse

        :param beneficiary_country: The two-letter ISO 3166-1 ALPHA-2 code for the country of the beneficiary’s bank account. The two-letter prefix of the payout method type matches the beneficiary country code., defaults to None
        :type beneficiary_country: str, optional
        :param beneficiary_currency: Currency accepted by the beneficiary’s bank account. Three-letter ISO 4217 code., defaults to None
        :type beneficiary_currency: str, optional
        :param beneficiary_entity_type: Type of entity for the beneficiary. One of the following:, defaults to None
        :type beneficiary_entity_type: HostedBeneficiaryTokenResponseBeneficiaryEntityType, optional
        :param beneficiary_extended_fields: When the value is **true**, the hosted tokenization page displays additional required fields.\<BR\> * Additional required fields when `beneficiary_entity_type` is **individual**: `address`, `city`, `country`, `date_of_birth`, `first_name`, `gender`, `identification_type`, `identification_value`, `last_name`, `nationality`. \<BR\> * Additional required fields when `beneficiary_entity_type` is **company**: `address`, `city`, `company_name`, `country`, `country_of_incorporation`, `date_of_incorporation`, `identification_type`, `identification_value`.\<BR\> For more information on the additional required fields, see 'Create Extended Beneficiary'.\<BR\> **Note**: Currently only available in the sandbox., defaults to None
        :type beneficiary_extended_fields: HostedBeneficiaryTokenResponseBeneficiaryExtendedFields, optional
        :param beneficiary_id: ID of the beneficiary. String starting with **beneficiary_**., defaults to None
        :type beneficiary_id: str, optional
        :param beneficiary_optional_fields: Additional information about the beneficiary., defaults to None
        :type beneficiary_optional_fields: List[HostedBeneficiaryTokenResponseBeneficiaryOptionalFields], optional
        :param beneficiary_validated: Indicates whether the beneficiary has been validated., defaults to None
        :type beneficiary_validated: bool, optional
        :param cancel_url: URL where the customer is redirected after pressing **Back to Website** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
        :type cancel_url: str, optional
        :param category: The category of payout method. One of the following: \<BR\> * **bank**\<BR\> * **card** - US only., defaults to None
        :type category: HostedBeneficiaryTokenResponseCategory, optional
        :param complete_url: URL where the customer is redirected after pressing **Close** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
        :type complete_url: str, optional
        :param country: The two-letter ISO 3166-1 ALPHA-2 code for the beneficiary’s country. The two-letter prefix of the payout method type must match the beneficiary country code., defaults to None
        :type country: str, optional
        :param currency: Currency accepted by the beneficiary’s bank account. Three-letter ISO 4217 code., defaults to None
        :type currency: str, optional
        :param entity_type: Type of entity for the beneficiary. One of the following:, defaults to None
        :type entity_type: HostedBeneficiaryTokenResponseEntityType, optional
        :param expiration: The page expiration date in Unix time., defaults to None
        :type expiration: str, optional
        :param id_: ID of the beneficiary tokenization page. String starting with **hp_ben_**.The page expiration date in Unix time., defaults to None
        :type id_: str, optional
        :param language: Determines the default language of the hosted page. For a list of values, see 'List Supported Languages'.\<BR\> * When this parameter is null, the language of the user's browser is used.\<BR\> * If the language of the user's browser cannot be determined, the default language is English., defaults to None
        :type language: str, optional
        :param merchant_alias: Client's name., defaults to None
        :type merchant_alias: str, optional
        :param merchant_color: Color of the call-to-action (CTA) button on the hosted page.\<BR\> To configure this field, use the Client Portal., defaults to None
        :type merchant_color: str, optional
        :param merchant_customer_support: Contains details of the client’s customer support. To configure these fields, use the Client Portal., defaults to None
        :type merchant_customer_support: List[HostedBeneficiaryTokenResponseMerchantCustomerSupport], optional
        :param merchant_logo: URL for the image of the client's logo.\<BR\> To configure this field, use the Client Portal., defaults to None
        :type merchant_logo: str, optional
        :param merchant_reference_id: Identifier defined by the client for reference purposes. Limit: 45 characters., defaults to None
        :type merchant_reference_id: str, optional
        :param merchant_website: The URL where the customer is redirected after exiting the hosted page. \<BR\> Relevant when one or both of the following fields is unset:\<BR\> * `cancel_url` \<BR\> * `complete_url`\<BR\> To configure this field, use the Client Portal., defaults to None
        :type merchant_website: str, optional
        :param page_expiration: End of the time when the customer can use the hosted page, in Unix time. If `page_expiration` is not set, the hosted page expires 14 days after creation. \<BR\> **Range**: 1 minute to 30 days., defaults to None
        :type page_expiration: float, optional
        :param payout_currency: Currency accepted by the beneficiary’s bank account. Three-letter ISO 4217 code., defaults to None
        :type payout_currency: str, optional
        :param payout_method_type: The payout method type selected by the customer. The two-letter prefix must match the beneficiary country code., defaults to None
        :type payout_method_type: str, optional
        :param payout_method_types_exclude: List of payout methods that are excluded from display on the beneficiary tokenization page., defaults to None
        :type payout_method_types_exclude: List[dict], optional
        :param payout_method_types_include: Array of strings. List of payout methods that are displayed on the beneficiary tokenization page., defaults to None
        :type payout_method_types_include: List[dict], optional
        :param redirect_url: URL of the hosted page that is shown to the customer., defaults to None
        :type redirect_url: str, optional
        :param sender_country: The two-letter ISO 3166-1 ALPHA-2 code for the sender’s country., defaults to None
        :type sender_country: str, optional
        :param sender_currency: Currency paid from the sender’s wallet. Three-letter ISO 4217 code., defaults to None
        :type sender_currency: str, optional
        :param sender_entity_type: Type of entity for the sender. One of the following:., defaults to None
        :type sender_entity_type: HostedBeneficiaryTokenResponseSenderEntityType, optional
        :param status: Status of the hosted page. One of the following:\<BR\>* **NEW** - The hosted page was created.\<BR\> * **DON** - Done. The beneficiary details were saved.\<BR\> * **EXP** - The hosted page expired., defaults to None
        :type status: HostedBeneficiaryTokenResponseStatus, optional
        :param timestamp: Timestamp for the request to create the beneficiary tokenization object page, in Unix time., defaults to None
        :type timestamp: str, optional
        :param tokenization_page: ID of the beneficiary tokenization page. String starting with **hp_ben_**., defaults to None
        :type tokenization_page: str, optional
        """
        self.beneficiary_country = self._define_str(
            "beneficiary_country", beneficiary_country, nullable=True
        )
        self.beneficiary_currency = self._define_str(
            "beneficiary_currency", beneficiary_currency, nullable=True
        )
        self.beneficiary_entity_type = (
            self._enum_matching(
                beneficiary_entity_type,
                HostedBeneficiaryTokenResponseBeneficiaryEntityType.list(),
                "beneficiary_entity_type",
            )
            if beneficiary_entity_type
            else None
        )
        self.beneficiary_extended_fields = (
            self._enum_matching(
                beneficiary_extended_fields,
                HostedBeneficiaryTokenResponseBeneficiaryExtendedFields.list(),
                "beneficiary_extended_fields",
            )
            if beneficiary_extended_fields
            else None
        )
        self.beneficiary_id = self._define_str(
            "beneficiary_id", beneficiary_id, nullable=True
        )
        self.beneficiary_optional_fields = self._define_list(
            beneficiary_optional_fields,
            HostedBeneficiaryTokenResponseBeneficiaryOptionalFields,
        )
        self.beneficiary_validated = beneficiary_validated
        self.cancel_url = self._define_str("cancel_url", cancel_url, nullable=True)
        self.category = (
            self._enum_matching(
                category, HostedBeneficiaryTokenResponseCategory.list(), "category"
            )
            if category
            else None
        )
        self.complete_url = self._define_str(
            "complete_url", complete_url, nullable=True
        )
        self.country = self._define_str("country", country, nullable=True)
        self.currency = self._define_str("currency", currency, nullable=True)
        self.entity_type = (
            self._enum_matching(
                entity_type,
                HostedBeneficiaryTokenResponseEntityType.list(),
                "entity_type",
            )
            if entity_type
            else None
        )
        self.expiration = self._define_str("expiration", expiration, nullable=True)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.language = self._define_str("language", language, nullable=True)
        self.merchant_alias = self._define_str(
            "merchant_alias", merchant_alias, nullable=True
        )
        self.merchant_color = self._define_str(
            "merchant_color", merchant_color, nullable=True
        )
        self.merchant_customer_support = self._define_list(
            merchant_customer_support,
            HostedBeneficiaryTokenResponseMerchantCustomerSupport,
        )
        self.merchant_logo = self._define_str(
            "merchant_logo", merchant_logo, nullable=True
        )
        self.merchant_reference_id = self._define_str(
            "merchant_reference_id", merchant_reference_id, nullable=True
        )
        self.merchant_website = self._define_str(
            "merchant_website", merchant_website, nullable=True
        )
        self.page_expiration = self._define_number(
            "page_expiration", page_expiration, nullable=True
        )
        self.payout_currency = self._define_str(
            "payout_currency", payout_currency, nullable=True
        )
        self.payout_method_type = self._define_str(
            "payout_method_type", payout_method_type, nullable=True
        )
        self.payout_method_types_exclude = payout_method_types_exclude
        self.payout_method_types_include = payout_method_types_include
        self.redirect_url = self._define_str(
            "redirect_url", redirect_url, nullable=True
        )
        self.sender_country = self._define_str(
            "sender_country", sender_country, nullable=True
        )
        self.sender_currency = self._define_str(
            "sender_currency", sender_currency, nullable=True
        )
        self.sender_entity_type = (
            self._enum_matching(
                sender_entity_type,
                HostedBeneficiaryTokenResponseSenderEntityType.list(),
                "sender_entity_type",
            )
            if sender_entity_type
            else None
        )
        self.status = (
            self._enum_matching(
                status, HostedBeneficiaryTokenResponseStatus.list(), "status"
            )
            if status
            else None
        )
        self.timestamp = self._define_str("timestamp", timestamp, nullable=True)
        self.tokenization_page = self._define_str(
            "tokenization_page", tokenization_page, nullable=True
        )



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class V1InvoicesBody(BaseModel):
    """V1InvoicesBody

    :param billing: Determines the method of billing at the end of the billing cycle. Set to **pay_automatically** - Rapyd generates a `payment` object, then attempts to pay it using the designated payment method.The number of days until the due date.
    :type billing: float
    :param currency: Three-letter ISO 4217 currency code for the currency used in all fields that refer to a monetary amount. Required when the subscription field is not provided. If subscription field is provided, then the currency defined in the subscription's plan is used., defaults to None
    :type currency: str, optional
    :param customer: ID of the customer that pays this invoice. String starting with **cus_**.
    :type customer: str
    :param days_until_due: The number of days until the due date., defaults to None
    :type days_until_due: float, optional
    :param description: Description of the invoice., defaults to None
    :type description: str, optional
    :param due_date: The date payment is due on this invoice. This value is calculated from the date the invoice is created, plus the number of days specified in the `days_until_due` field. Relevant when `billing` is **send_invoice**.  Format is in Unix time., defaults to None
    :type due_date: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param payment_fields: Object containing additional payment_options fields., defaults to None
    :type payment_fields: dict, optional
    :param payment_method: ID of the payment method for paying the invoice. If not provided, then the payment method is taken from the subscription. If the payment method is not provided in the subscription, the payment method is the customer's default payment method., defaults to None
    :type payment_method: str, optional
    :param statement_descriptor: Description of the invoice for the customer's credit card statement. Limited to 22 characters., defaults to None
    :type statement_descriptor: str, optional
    :param subscription: ID of the subscription that generates charges to this customer. String starting with **sub_**., defaults to None
    :type subscription: str, optional
    :param tax_percent: The tax rate, defined as a percentage., defaults to None
    :type tax_percent: float, optional
    """

    def __init__(
        self,
        billing: float,
        customer: str,
        currency: str = None,
        days_until_due: float = None,
        description: str = None,
        due_date: str = None,
        metadata: dict = None,
        payment_fields: dict = None,
        payment_method: str = None,
        statement_descriptor: str = None,
        subscription: str = None,
        tax_percent: float = None,
    ):
        """V1InvoicesBody

        :param billing: Determines the method of billing at the end of the billing cycle. Set to **pay_automatically** - Rapyd generates a `payment` object, then attempts to pay it using the designated payment method.The number of days until the due date.
        :type billing: float
        :param currency: Three-letter ISO 4217 currency code for the currency used in all fields that refer to a monetary amount. Required when the subscription field is not provided. If subscription field is provided, then the currency defined in the subscription's plan is used., defaults to None
        :type currency: str, optional
        :param customer: ID of the customer that pays this invoice. String starting with **cus_**.
        :type customer: str
        :param days_until_due: The number of days until the due date., defaults to None
        :type days_until_due: float, optional
        :param description: Description of the invoice., defaults to None
        :type description: str, optional
        :param due_date: The date payment is due on this invoice. This value is calculated from the date the invoice is created, plus the number of days specified in the `days_until_due` field. Relevant when `billing` is **send_invoice**.  Format is in Unix time., defaults to None
        :type due_date: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param payment_fields: Object containing additional payment_options fields., defaults to None
        :type payment_fields: dict, optional
        :param payment_method: ID of the payment method for paying the invoice. If not provided, then the payment method is taken from the subscription. If the payment method is not provided in the subscription, the payment method is the customer's default payment method., defaults to None
        :type payment_method: str, optional
        :param statement_descriptor: Description of the invoice for the customer's credit card statement. Limited to 22 characters., defaults to None
        :type statement_descriptor: str, optional
        :param subscription: ID of the subscription that generates charges to this customer. String starting with **sub_**., defaults to None
        :type subscription: str, optional
        :param tax_percent: The tax rate, defined as a percentage., defaults to None
        :type tax_percent: float, optional
        """
        self.billing = billing
        self.currency = self._define_str("currency", currency, nullable=True)
        self.customer = customer
        self.days_until_due = self._define_number(
            "days_until_due", days_until_due, nullable=True
        )
        self.description = self._define_str("description", description, nullable=True)
        self.due_date = self._define_str("due_date", due_date, nullable=True)
        self.metadata = metadata
        self.payment_fields = payment_fields
        self.payment_method = self._define_str(
            "payment_method", payment_method, nullable=True
        )
        self.statement_descriptor = self._define_str(
            "statement_descriptor", statement_descriptor, nullable=True
        )
        self.subscription = self._define_str(
            "subscription", subscription, nullable=True
        )
        self.tax_percent = self._define_number(
            "tax_percent", tax_percent, nullable=True
        )



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class CardsReversalBody(BaseModel):
    """CardsReversalBody

    :param amount: The amount of the authorization, in units of the `currency` defined in currency. Decimal., defaults to None
    :type amount: float, optional
    :param authorization_id: ID of the authorization. String starting with **cardauth_**. Use the value of `id` in the response to Simulate a Card Transaction Authorization Request - EEA.
    :type authorization_id: str
    :param card_id: ID of the card. String starting with **card_**.
    :type card_id: str
    :param currency: Defines the currency for the transaction. Three-letter ISO 4217 code.
    :type currency: str
    """

    def __init__(
        self, authorization_id: str, card_id: str, currency: str, amount: float = None
    ):
        """CardsReversalBody

        :param amount: The amount of the authorization, in units of the `currency` defined in currency. Decimal., defaults to None
        :type amount: float, optional
        :param authorization_id: ID of the authorization. String starting with **cardauth_**. Use the value of `id` in the response to Simulate a Card Transaction Authorization Request - EEA.
        :type authorization_id: str
        :param card_id: ID of the card. String starting with **card_**.
        :type card_id: str
        :param currency: Defines the currency for the transaction. Three-letter ISO 4217 code.
        :type currency: str
        """
        self.amount = self._define_number("amount", amount, nullable=True)
        self.authorization_id = authorization_id
        self.card_id = card_id
        self.currency = currency



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class VerifyHostedAppResponseMerchantDetailsMerchantCustomerSupport(BaseModel):
    """VerifyHostedAppResponseMerchantDetailsMerchantCustomerSupport

    :param email: email, defaults to None
    :type email: str, optional
    :param url: url, defaults to None
    :type url: str, optional
    :param phone_number: phone_number, defaults to None
    :type phone_number: str, optional
    """

    def __init__(self, email: str = None, url: str = None, phone_number: str = None):
        """VerifyHostedAppResponseMerchantDetailsMerchantCustomerSupport

        :param email: email, defaults to None
        :type email: str, optional
        :param url: url, defaults to None
        :type url: str, optional
        :param phone_number: phone_number, defaults to None
        :type phone_number: str, optional
        """
        self.email = self._define_str("email", email, nullable=True)
        self.url = self._define_str("url", url, nullable=True)
        self.phone_number = self._define_str(
            "phone_number", phone_number, nullable=True
        )



from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .subscription_item import SubscriptionItem


@JsonMap({})
class SubscriptionItems(BaseModel):
    """SubscriptionItems

    :param data: A list of up to three subscription items., defaults to None
    :type data: List[SubscriptionItem], optional
    :param has_more: Indicates whether there are more than three items in this subscription., defaults to None
    :type has_more: bool, optional
    :param total_count: Total number of items in this subscription., defaults to None
    :type total_count: float, optional
    :param url: URL for requesting all of the items in this subscription., defaults to None
    :type url: str, optional
    """

    def __init__(
        self,
        data: List[SubscriptionItem] = None,
        has_more: bool = None,
        total_count: float = None,
        url: str = None,
    ):
        """SubscriptionItems

        :param data: A list of up to three subscription items., defaults to None
        :type data: List[SubscriptionItem], optional
        :param has_more: Indicates whether there are more than three items in this subscription., defaults to None
        :type has_more: bool, optional
        :param total_count: Total number of items in this subscription., defaults to None
        :type total_count: float, optional
        :param url: URL for requesting all of the items in this subscription., defaults to None
        :type url: str, optional
        """
        self.data = self._define_list(data, SubscriptionItem)
        self.has_more = has_more
        self.total_count = self._define_number(
            "total_count", total_count, nullable=True
        )
        self.url = self._define_str("url", url, nullable=True)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({"id_": "id"})
class InlineResponse200_69Data(BaseModel):
    """InlineResponse200_69Data

    :param id_: id_, defaults to None
    :type id_: str, optional
    :param quantity: The usage quantity for the specified date and time (timestamp)., defaults to None
    :type quantity: float, optional
    :param subscription_item: ID of the subscription item., defaults to None
    :type subscription_item: str, optional
    :param timestamp: Timestamp for the usage record in Unix time., defaults to None
    :type timestamp: str, optional
    """

    def __init__(
        self,
        id_: str = None,
        quantity: float = None,
        subscription_item: str = None,
        timestamp: str = None,
    ):
        """InlineResponse200_69Data

        :param id_: id_, defaults to None
        :type id_: str, optional
        :param quantity: The usage quantity for the specified date and time (timestamp)., defaults to None
        :type quantity: float, optional
        :param subscription_item: ID of the subscription item., defaults to None
        :type subscription_item: str, optional
        :param timestamp: Timestamp for the usage record in Unix time., defaults to None
        :type timestamp: str, optional
        """
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.quantity = self._define_number("quantity", quantity, nullable=True)
        self.subscription_item = self._define_str(
            "subscription_item", subscription_item, nullable=True
        )
        self.timestamp = self._define_str("timestamp", timestamp, nullable=True)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class PaymentEwallets(BaseModel):
    """PaymentEwallets

    :param ewallet_id: ewallet_id, defaults to None
    :type ewallet_id: str, optional
    :param amount: amount, defaults to None
    :type amount: float, optional
    :param percent: percent, defaults to None
    :type percent: float, optional
    :param refunded_amount: refunded_amount, defaults to None
    :type refunded_amount: float, optional
    :param released_amount: released_amount, defaults to None
    :type released_amount: float, optional
    """

    def __init__(
        self,
        ewallet_id: str = None,
        amount: float = None,
        percent: float = None,
        refunded_amount: float = None,
        released_amount: float = None,
    ):
        """PaymentEwallets

        :param ewallet_id: ewallet_id, defaults to None
        :type ewallet_id: str, optional
        :param amount: amount, defaults to None
        :type amount: float, optional
        :param percent: percent, defaults to None
        :type percent: float, optional
        :param refunded_amount: refunded_amount, defaults to None
        :type refunded_amount: float, optional
        :param released_amount: released_amount, defaults to None
        :type released_amount: float, optional
        """
        self.ewallet_id = self._define_str("ewallet_id", ewallet_id, nullable=True)
        self.amount = self._define_number("amount", amount, nullable=True)
        self.percent = self._define_number("percent", percent, nullable=True)
        self.refunded_amount = self._define_number(
            "refunded_amount", refunded_amount, nullable=True
        )
        self.released_amount = self._define_number(
            "released_amount", released_amount, nullable=True
        )



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .card_token_response_card_fields import CardTokenResponseCardFields
from .merchant_customer_support import MerchantCustomerSupport
from .card_token_response_payment_params import CardTokenResponsePaymentParams


@JsonMap({"id_": "id"})
class CardTokenResponse(BaseModel):
    """CardTokenResponse

    :param billing_address_collect: Indicates whether the hosted page displays address fields to fill in.\<BR\> * **true** - The address fields are displayed. \<BR\> * **false** - The address fields are displayed only for country codes **US**, **GB** and **CA**., defaults to None
    :type billing_address_collect: bool, optional
    :param cancel_url: URL where the customer is redirected after pressing **Back to Website** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
    :type cancel_url: str, optional
    :param card_fields: Contains details about the card., defaults to None
    :type card_fields: CardTokenResponseCardFields, optional
    :param category: Category of payment method: **card**., defaults to None
    :type category: str, optional
    :param complete_url: URL where the customer is redirected after pressing **Close** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs.The two-letter ISO 3166-1 ALPHA-2 code for the country., defaults to None
    :type complete_url: str, optional
    :param complete_payment_url: URL where the customer is redirected when payment is successful, after returning from an external page such as a 3DS page. Does not support localhost URLs., defaults to None
    :type complete_payment_url: str, optional
    :param country: The two-letter ISO 3166-1 ALPHA-2 code for the country., defaults to None
    :type country: str, optional
    :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code. \<BR\> In FX transactions: \<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet. \<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer. \<BR\> See also `fixed_side` and `requested_currency` fields., defaults to None
    :type currency: str, optional
    :param customer: ID of a specific customer. String starting with **cus_**., defaults to None
    :type customer: str, optional
    :param error_code: Relevant error message and ID number of the error., defaults to None
    :type error_code: str, optional
    :param error_payment_url: URL where the customer is redirected when payment is not successful, after returning from an external page, such as a 3DS page. Does not support localhost URLs., defaults to None
    :type error_payment_url: str, optional
    :param id_: ID of the card token hosted page, a string starting with **hp_card_**., defaults to None
    :type id_: str, optional
    :param language: Determines the default language of the hosted page. For a list of values, see [List Supported Languages](https://docs.rapyd.net/en/list-supported-languages.html). \<BR\> * When this parameter is null, the language of the user's browser is used. \<BR\> If the language of the user's browser cannot be determined, the default language is English., defaults to None
    :type language: str, optional
    :param merchant_alias: Reserved., defaults to None
    :type merchant_alias: str, optional
    :param merchant_color: Color of the call-to-action (CTA) button on the hosted page. To configure this field, use the Client Portal. See [Customizing Your Hosted Page](https://docs.rapyd.net/en/customizing-your-hosted-page.html)., defaults to None
    :type merchant_color: str, optional
    :param merchant_customer_support: Contains details of the client’s customer support. To configure these fields, use the Client Portal., defaults to None
    :type merchant_customer_support: MerchantCustomerSupport, optional
    :param merchant_website: The URL where the customer is redirected after exiting the hosted page. Relevant when one or both of the following fields is unset: \<BR\>* `cancel_url` \<BR\>* `complete_url`.\<BR\> To configure this field, use the **Fallback URL** field in the Client Portal. See [Customizing Your Hosted Page](https://docs.rapyd.net/en/customizing-your-hosted-page.html)., defaults to None
    :type merchant_website: str, optional
    :param page_expiration: End of the time when the customer can use the hosted page, in Unix time. If `page_expiration` is not set, the hosted page expires 14 days after creation. Range: 1 minute to 30 days., defaults to None
    :type page_expiration: str, optional
    :param payment_method_type: Limits the page to a specific type of payment method. For example, **dk_visa_card**. For a list of payment methods, use []List Payment Methods by Country](https://docs.rapyd.net/en/list-payment-methods-by-country.html)., defaults to None
    :type payment_method_type: str, optional
    :param payment_params: Contains the following fields. When these fields do not include values, the user is redirected to the hosted page, and a related status message appears at the top of the page., defaults to None
    :type payment_params: CardTokenResponsePaymentParams, optional
    :param redirect_url: URL of the hosted page that is shown to the customer.Contains the following fields. When these fields do not include values, the user is redirected to the hosted page, and a related status message appears at the top of the page., defaults to None
    :type redirect_url: str, optional
    :param status: Status of the hosted page. One of the following: \<BR\> * **NEW** - The hosted page was created. \<BR\> * **DON** - Done. The card was added to the customer profile. \<BR\> * **EXP** - The hosted page expired.URL of the hosted page that is shown to the customer., defaults to None
    :type status: str, optional
    """

    def __init__(
        self,
        billing_address_collect: bool = None,
        cancel_url: str = None,
        card_fields: CardTokenResponseCardFields = None,
        category: str = None,
        complete_url: str = None,
        complete_payment_url: str = None,
        country: str = None,
        currency: str = None,
        customer: str = None,
        error_code: str = None,
        error_payment_url: str = None,
        id_: str = None,
        language: str = None,
        merchant_alias: str = None,
        merchant_color: str = None,
        merchant_customer_support: MerchantCustomerSupport = None,
        merchant_website: str = None,
        page_expiration: str = None,
        payment_method_type: str = None,
        payment_params: CardTokenResponsePaymentParams = None,
        redirect_url: str = None,
        status: str = None,
    ):
        """CardTokenResponse

        :param billing_address_collect: Indicates whether the hosted page displays address fields to fill in.\<BR\> * **true** - The address fields are displayed. \<BR\> * **false** - The address fields are displayed only for country codes **US**, **GB** and **CA**., defaults to None
        :type billing_address_collect: bool, optional
        :param cancel_url: URL where the customer is redirected after pressing **Back to Website** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
        :type cancel_url: str, optional
        :param card_fields: Contains details about the card., defaults to None
        :type card_fields: CardTokenResponseCardFields, optional
        :param category: Category of payment method: **card**., defaults to None
        :type category: str, optional
        :param complete_url: URL where the customer is redirected after pressing **Close** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs.The two-letter ISO 3166-1 ALPHA-2 code for the country., defaults to None
        :type complete_url: str, optional
        :param complete_payment_url: URL where the customer is redirected when payment is successful, after returning from an external page such as a 3DS page. Does not support localhost URLs., defaults to None
        :type complete_payment_url: str, optional
        :param country: The two-letter ISO 3166-1 ALPHA-2 code for the country., defaults to None
        :type country: str, optional
        :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code. \<BR\> In FX transactions: \<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet. \<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer. \<BR\> See also `fixed_side` and `requested_currency` fields., defaults to None
        :type currency: str, optional
        :param customer: ID of a specific customer. String starting with **cus_**., defaults to None
        :type customer: str, optional
        :param error_code: Relevant error message and ID number of the error., defaults to None
        :type error_code: str, optional
        :param error_payment_url: URL where the customer is redirected when payment is not successful, after returning from an external page, such as a 3DS page. Does not support localhost URLs., defaults to None
        :type error_payment_url: str, optional
        :param id_: ID of the card token hosted page, a string starting with **hp_card_**., defaults to None
        :type id_: str, optional
        :param language: Determines the default language of the hosted page. For a list of values, see [List Supported Languages](https://docs.rapyd.net/en/list-supported-languages.html). \<BR\> * When this parameter is null, the language of the user's browser is used. \<BR\> If the language of the user's browser cannot be determined, the default language is English., defaults to None
        :type language: str, optional
        :param merchant_alias: Reserved., defaults to None
        :type merchant_alias: str, optional
        :param merchant_color: Color of the call-to-action (CTA) button on the hosted page. To configure this field, use the Client Portal. See [Customizing Your Hosted Page](https://docs.rapyd.net/en/customizing-your-hosted-page.html)., defaults to None
        :type merchant_color: str, optional
        :param merchant_customer_support: Contains details of the client’s customer support. To configure these fields, use the Client Portal., defaults to None
        :type merchant_customer_support: MerchantCustomerSupport, optional
        :param merchant_website: The URL where the customer is redirected after exiting the hosted page. Relevant when one or both of the following fields is unset: \<BR\>* `cancel_url` \<BR\>* `complete_url`.\<BR\> To configure this field, use the **Fallback URL** field in the Client Portal. See [Customizing Your Hosted Page](https://docs.rapyd.net/en/customizing-your-hosted-page.html)., defaults to None
        :type merchant_website: str, optional
        :param page_expiration: End of the time when the customer can use the hosted page, in Unix time. If `page_expiration` is not set, the hosted page expires 14 days after creation. Range: 1 minute to 30 days., defaults to None
        :type page_expiration: str, optional
        :param payment_method_type: Limits the page to a specific type of payment method. For example, **dk_visa_card**. For a list of payment methods, use []List Payment Methods by Country](https://docs.rapyd.net/en/list-payment-methods-by-country.html)., defaults to None
        :type payment_method_type: str, optional
        :param payment_params: Contains the following fields. When these fields do not include values, the user is redirected to the hosted page, and a related status message appears at the top of the page., defaults to None
        :type payment_params: CardTokenResponsePaymentParams, optional
        :param redirect_url: URL of the hosted page that is shown to the customer.Contains the following fields. When these fields do not include values, the user is redirected to the hosted page, and a related status message appears at the top of the page., defaults to None
        :type redirect_url: str, optional
        :param status: Status of the hosted page. One of the following: \<BR\> * **NEW** - The hosted page was created. \<BR\> * **DON** - Done. The card was added to the customer profile. \<BR\> * **EXP** - The hosted page expired.URL of the hosted page that is shown to the customer., defaults to None
        :type status: str, optional
        """
        self.billing_address_collect = billing_address_collect
        self.cancel_url = self._define_str("cancel_url", cancel_url, nullable=True)
        self.card_fields = self._define_object(card_fields, CardTokenResponseCardFields)
        self.category = self._define_str("category", category, nullable=True)
        self.complete_url = self._define_str(
            "complete_url", complete_url, nullable=True
        )
        self.complete_payment_url = self._define_str(
            "complete_payment_url", complete_payment_url, nullable=True
        )
        self.country = self._define_str("country", country, nullable=True)
        self.currency = self._define_str("currency", currency, nullable=True)
        self.customer = self._define_str("customer", customer, nullable=True)
        self.error_code = self._define_str("error_code", error_code, nullable=True)
        self.error_payment_url = self._define_str(
            "error_payment_url", error_payment_url, nullable=True
        )
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.language = self._define_str("language", language, nullable=True)
        self.merchant_alias = self._define_str(
            "merchant_alias", merchant_alias, nullable=True
        )
        self.merchant_color = self._define_str(
            "merchant_color", merchant_color, nullable=True
        )
        self.merchant_customer_support = self._define_object(
            merchant_customer_support, MerchantCustomerSupport
        )
        self.merchant_website = self._define_str(
            "merchant_website", merchant_website, nullable=True
        )
        self.page_expiration = self._define_str(
            "page_expiration", page_expiration, nullable=True
        )
        self.payment_method_type = self._define_str(
            "payment_method_type", payment_method_type, nullable=True
        )
        self.payment_params = self._define_object(
            payment_params, CardTokenResponsePaymentParams
        )
        self.redirect_url = self._define_str(
            "redirect_url", redirect_url, nullable=True
        )
        self.status = self._define_str("status", status, nullable=True)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class SimulateClearingCardTransactionEeaRemoteAuthResponse(BaseModel):
    """Response to a successful remote authorization request. Contains the following fields:

    :param authorization_id: ID of the authorization. String starting with **cardauth_**., defaults to None
    :type authorization_id: str, optional
    :param response_code: Code returned by the client in the response to the remote authorization., defaults to None
    :type response_code: str, optional
    :param auth_code: Authorization code returned by the client in the response to the remote authorization., defaults to None
    :type auth_code: str, optional
    """

    def __init__(
        self,
        authorization_id: str = None,
        response_code: str = None,
        auth_code: str = None,
    ):
        """Response to a successful remote authorization request. Contains the following fields:

        :param authorization_id: ID of the authorization. String starting with **cardauth_**., defaults to None
        :type authorization_id: str, optional
        :param response_code: Code returned by the client in the response to the remote authorization., defaults to None
        :type response_code: str, optional
        :param auth_code: Authorization code returned by the client in the response to the remote authorization., defaults to None
        :type auth_code: str, optional
        """
        self.authorization_id = self._define_str(
            "authorization_id", authorization_id, nullable=True
        )
        self.response_code = self._define_str(
            "response_code", response_code, nullable=True
        )
        self.auth_code = self._define_str("auth_code", auth_code, nullable=True)
